# THIS FILE IS AUTO-GENERATED - DO NOT EDIT
STACKS_PAYWALL_TEMPLATE = "<!DOCTYPE html>\n<html lang=\"en\">\n<head>\n    <meta charset=\"UTF-8\">\n    <meta name=\"viewport\" content=\"width=device-width, initial-scale=1.0\">\n    <title>Payment Required</title>\n    <link rel=\"stylesheet\" href=\"https://cdn.jsdelivr.net/npm/@t402/paywall@2.4.0/dist/browser/stacks.min.css\">\n</head>\n<body>\n    <div id=\"root\"></div>\n    <script src=\"https://cdn.jsdelivr.net/npm/@t402/paywall@2.4.0/dist/browser/stacks.min.js\"></script>\n</body>\n</html>"
